pub struct Compactor { /* private fields */ }Expand description
Compacts context by generating an LLM summary
Implementations
Sourceimpl Compactor
impl Compactor
pub fn new(llm: Arc<dyn StreamingModelProvider>) -> Self
Sourcepub async fn compact(
&self,
context: &Context,
) -> Result<CompactionResult, CompactionError>
pub async fn compact( &self, context: &Context, ) -> Result<CompactionResult, CompactionError>
Generate a structured summary of the conversation and return a new compacted context.
This is a pure function that takes a reference to the context and returns a new context with the compacted messages replaced by a summary.