pub enum ToolEvent {
Call {
request: ToolCallRequest,
},
CallUpdate {
tool_call_id: String,
chunk: String,
},
ExecutionStarted {
tool_id: String,
tool_name: String,
},
Progress {
request: ToolCallRequest,
progress: f64,
total: Option<f64>,
message: Option<String>,
},
Result {
result: ToolCallResult,
result_meta: Option<ToolResultMeta>,
},
Error {
error: ToolCallError,
},
DefinitionsUpdated {
tools: Vec<ToolDefinition>,
},
}Expand description
Tool call lifecycle events.
Variants
Call
The LLM requested a tool call; arguments may still be streaming.
Fields

request: ToolCallRequestCallUpdate
A chunk of streamed tool call arguments.
ExecutionStarted
The tool began executing.
Progress
Progress reported by an executing tool.
Result
The tool completed successfully.
Error
The tool failed.
Fields

error: ToolCallErrorDefinitionsUpdated
The set of available tool definitions changed.
Fields

tools: Vec<ToolDefinition>Trait Implementations
Sourceimpl<'de> Deserialize<'de> for ToolEvent
impl<'de> Deserialize<'de> for ToolEvent
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
Sourceimpl JsonSchema for ToolEvent
impl JsonSchema for ToolEvent
Sourcefn schema_id() -> Cow<'static, str>
fn schema_id() -> Cow<'static, str>
Returns a string that uniquely identifies the schema produced by this type. Read more
Sourcefn json_schema(generator: &mut SchemaGenerator) -> Schema
fn json_schema(generator: &mut SchemaGenerator) -> Schema
Generates a JSON Schema for this type. Read more
Sourcefn inline_schema() -> bool
fn inline_schema() -> bool
Whether JSON Schemas generated for this type should be included directly in parent schemas,
rather than being re-used where possible using the
$ref keyword. Read moreimpl StructuralPartialEq for ToolEvent